import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;


/** Hauptservlet der Artikelveraltung
  * Hierin wird die Artikelliste erzeugt, die Artikel gelscht und gespeichert
  *
  * @author Bernhard Fiser 0109815
  * @version 1.1
  */
public class ArtikelVerwaltungServlet extends HttpServlet
{
	private static ArtikelVerwaltung
			av;

	/** Servletinitialisierung beim erstmaligen Laden des Servlets
          * vom Tomcat */
	public void init()
	{
		av = ArtikelVerwaltung.getInstance();
	}

	/** Methode fr CGI-POST. Diese wird auf die doGet Methode
	  * weitergeleitet */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
	{
		this.doGet(request, response);
	}

	/** Methode fr CGI-GET. Hier findet die Verarbeitung der Daten statt */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
	{
		LinkedList
			l;
		int	c,
			idn;
		String	id,
			bg,
			desc;
		Art	art;

		/* Contenttype fr output setzen */
		response.setContentType("text/html");
		/* Outputstream festhalten */
		PrintWriter out = response.getWriter();
		/* HTML-Doc Einleitung */
		out.println(
			"<html><head><meta name=\"author\" content=\"Bernhard Fiser\">" +
			"<meta name=\"date\" content=\"2002-05-14T08:28:00+02:00\">" +
			"<title>Artikelverwalung V1.0 - Bernhard Fiser 0109815</title></head><body>\n");

		/* Falls der CGI-Parameter 'cgi_save' bergeben wurde,
		 * Artikel in Artikelverwaltung aufnehmen und speichern */
		if (request.getParameter("cgi_save") != null)
		{
			id = request.getParameter("cgi_art");
			art = (Art) av.getArten().get(id.toLowerCase());
			try
			{
				/* CGI-Parameter aus FORM bernehmen und convertiertn */
				idn = Integer.parseInt(request.getParameter("cgi_id"));
			}
			catch (NumberFormatException e)
			{
				/* Falls Konvertierung fehlgeschlagen, Werte auf 0 setzen */
				idn = 0;
			}
			/* restliche CGI-Parameter bernehmen. */
			desc = request.getParameter("cgi_desc");

			/* int Werte auf Plausibilitt prfen */
			if ((idn > 0) && (art != null))
			{
				/* Falls Werte in Ordnung, neuen Artikel anlegen... */
				Artikel a = new Artikel((Art) av.getArten().get("bo"));
				if (a.parseLine(art.getDescription() + ";" + desc + ";" + art.getID() + idn, av.getArten()))
				{
					/* ... und in Artikelverwaltun aufnehmen... */
					av.remove(a.getID().toLowerCase());
					av.put(a.getID().toLowerCase(), a);
					/* ... und speichern */
					if (!av.save())
						out.println("<font color=\"red\"><b>Fataler Fehler beim Speichern!</b></font><br>");
					else
						/* Falls Speicherung erfolgreich, dann OK-Meldung ausgeben */
						out.println("<font color=\"green\">Artikel Nummer " + a.getID() + " wurde erfolgreich gespeichert!</font><br>");
				}
				else
					/* Fataler Fehler (drfte eigentlich nicht vorkommen) */
					out.println("<font color=\"red\">*** fatal: Artikel konnte nicht angelegt werden! Eingabedaten sind falsch.</font><br>");
			}
			else
				/* Falls Plausibilittsprfung fehlgeschlagen, Fehler ausgeben */
				out.println("<font color=\"red\">Artikel konnte nicht gespeichert werden! Eingabedaten sind falsch.</font><br>");
		}

		/* Falls CGI-Parameter 'cgi_del' bergeben wurde, Artikelobjekt
		 * aus der Artikelverwaltung lschen */
		if (request.getParameter("cgi_del") != null)
		{
			/* bergebene Artikelnummer konvertieren und auf
			 * Plausibilitt prfen */
			id = request.getParameter("cgi_id");
			/* Falls Plausibilitt OK, Artikel lschen */
			if (av.remove(id.toLowerCase()) != null)
			{
				/* ArtikelVerwaltung speichern */
				if (!av.save())
					out.println("<font color=\"red\"><b>Fataler Fehler beim Speichern!</b></font><br>");
				else
					/* OK-Meldung ausgeben, falls Speicherung erfolgreich */
					out.println("<font color=\"green\">Artikel Nummer " + id + " wurde erfolgreich gel&ouml;scht.</font><br>");
			}
			else
				/* Falls Plausibilittsprfung fehlgeschlagen, Fehler ausgeben */
				out.println("<font color=\"red\"Artikel Nummer " + id + " konnte nicht gel&ouml;scht werden. Artikel existiert nicht.</font></br>");
		}

		/* Button fr Neuanlage generiern*/
		out.println("<form method=\"post\" action=\"ArtikelDetailServlet\"><input type=\"submit\" value=\"Neuen Artikel anlegen\"></form><hr>");
		/* Tabelle der vorhandenen Artikel generieren */
		out.println("<table border=\"1\"><tr><th>ID</th><th>Beschreibung</th><th>Art</th></tr>");
		/* Iterator der Hashmap erstellen */
		l = sortArtikelVerwaltung();
		ListIterator it = l.listIterator();
		c = 0;
		/* ber jedes Objekt iterieren */
		while (it.hasNext())
		{
			/* Hintergrundfarbe der Tabelle festlegen */
			if ((c++ % 2) == 0)
				bg = "#aaeeaa";
			else
				bg = "#eeeeaa";
			/* Artikel auslesen aus der Hashmap */
			Artikel a = (Artikel) it.next();
			/* Zeile mit den Artikelattribute ausgeben */
			out.println(
				"<tr bgcolor=\"" + bg + "\">" +
				"<td>" + a.getID() + "</td>" +
				"<td>" + a.getDescription() + "</td>" +
				"<td>" + a.getArt().getDescription() + "</td>" +
				"<td><a href=\"ArtikelDetailServlet?cgi_id=" + a.getID() + "\">&Auml;ndern</a></td>" +
				"<td><a href=\"ArtikelVerwaltungServlet?cgi_id=" + a.getID() + "&cgi_del=1\">L&ouml;schen</a></td>" +
				"</tr>\n");
		}
		/* Tabelle und HTML-Doc beenden */
		out.println("</table></body></html>\n");
	}

	/** Sortierung der Artikelverwaltung fr die Darstellung am Bildschirm */
	private LinkedList sortArtikelVerwaltung()
	{
		Artikel		a, b;
		LinkedList	l;
		ListIterator	j;
		Iterator	k;
		int		i;

		/* neue LinkedList erstellen und Iterator fr Hashmap erstellen */
		l = new LinkedList();
		k = av.values().iterator();
		/* Iteration ber alle Elemente der Artikelverwaltung */
		while (k.hasNext())
		{
			/* Artikel aus Hashmap lesen */
			a = (Artikel) k.next();
			i = 0;
			/* Iteration ber Linkedlist */
			j = l.listIterator(0);
			while (j.hasNext())
			{
				/* Artikel aus LinkedList lesen */
				b = (Artikel) j.next();
				/* Falls Artikel der Liste grere Artikelnummer hat, als
				 * Artikel aus Hashmap */
				if (b.getID().compareToIgnoreCase(a.getID()) > 0)
					/* ... dann Schleife abbrechen */
					break;
				i++;
			}
			/* Artikel aus Hashmap an der Stelle in die LinkedList
			 * einsortieren, an der die Schleife abgebrochen wurde (s.o.) */
			l.add(i, a);
		}
		/* LinkedList zurckgehen */
		return(l);
	}
}
